#!/bin/bash

# Parse arguments. Generated by parse-options.sh.
_new_arguments=()
while [[ $# -gt 0 ]]; do
    case "$1" in
        --help) help=1; shift ;;
        --version) version=1; shift ;;
        --certificate-name=*) certificate_name="${1#*=}"; shift ;;
        --certificate-name) if [[ ! $2 == "" && ! $2 =~ (^--$|^-[^-]|^--[^-]) ]]; then certificate_name="$2"; shift; fi; shift ;;
        --fast) fast=1; shift ;;
        --no-auto-add-group) no_auto_add_group=1; shift ;;
        --no-drush-install) no_drush_install=1; shift ;;
        --no-sites-default) no_sites_default=1; shift ;;
        --offline) offline=1; shift ;;
        --php-fpm-config=*) php_fpm_config+=("${1#*=}"); shift ;;
        --php-fpm-config) if [[ ! $2 == "" && ! $2 =~ (^--$|^-[^-]|^--[^-]) ]]; then php_fpm_config+=("$2"); shift; fi; shift ;;
        --php-fpm-user=*) php_fpm_user="${1#*=}"; shift ;;
        --php-fpm-user) if [[ ! $2 == "" && ! $2 =~ (^--$|^-[^-]|^--[^-]) ]]; then php_fpm_user="$2"; shift; fi; shift ;;
        --prefix=*) prefix="${1#*=}"; shift ;;
        --prefix) if [[ ! $2 == "" && ! $2 =~ (^--$|^-[^-]|^--[^-]) ]]; then prefix="$2"; shift; fi; shift ;;
        --project-container=*) project_container="${1#*=}"; shift ;;
        --project-container) if [[ ! $2 == "" && ! $2 =~ (^--$|^-[^-]|^--[^-]) ]]; then project_container="$2"; shift; fi; shift ;;
        --project-name=*) project_name="${1#*=}"; shift ;;
        --project-name) if [[ ! $2 == "" && ! $2 =~ (^--$|^-[^-]|^--[^-]) ]]; then project_name="$2"; shift; fi; shift ;;
        --project-parent-name=*) project_parent_name="${1#*=}"; shift ;;
        --project-parent-name) if [[ ! $2 == "" && ! $2 =~ (^--$|^-[^-]|^--[^-]) ]]; then project_parent_name="$2"; shift; fi; shift ;;
        --url=*) url="${1#*=}"; shift ;;
        --url) if [[ ! $2 == "" && ! $2 =~ (^--$|^-[^-]|^--[^-]) ]]; then url="$2"; shift; fi; shift ;;
        --variation=*) variation="${1#*=}"; shift ;;
        --variation) if [[ ! $2 == "" && ! $2 =~ (^--$|^-[^-]|^--[^-]) ]]; then variation="$2"; shift; fi; shift ;;
        --with-certbot-obtain) certbot_obtain=1; shift ;;
        --without-certbot-obtain) certbot_obtain=0; shift ;;
        --[^-]*) shift ;;
        *) _new_arguments+=("$1"); shift ;;
    esac
done
set -- "${_new_arguments[@]}"
unset _new_arguments

# Common Functions.
red() { echo -ne "\e[91m" >&2; echo -n "$@" >&2; echo -ne "\e[39m" >&2; }
green() { echo -ne "\e[92m" >&2; echo -n "$@" >&2; echo -ne "\e[39m" >&2; }
yellow() { echo -ne "\e[93m" >&2; echo -n "$@" >&2; echo -ne "\e[39m" >&2; }
blue() { echo -ne "\e[94m" >&2; echo -n "$@" >&2; echo -ne "\e[39m" >&2; }
magenta() { echo -ne "\e[95m" >&2; echo -n "$@" >&2; echo -ne "\e[39m" >&2; }
error() { echo -n "$INDENT" >&2; red '#' "$@" >&2; echo >&2; }
success() { echo -n "$INDENT" >&2; green '#' "$@" >&2; echo >&2; }
chapter() { echo -n "$INDENT" >&2; yellow '#' "$@" >&2; echo >&2; }
title() { echo -n "$INDENT" >&2; blue '#' "$@" >&2; echo >&2; }
code() { echo -n "$INDENT" >&2; magenta "$@" >&2; echo >&2; }
x() { echo >&2; exit 1; }
e() { echo -n "$INDENT" >&2; echo -n "$@" >&2; }
_() { echo -n "$INDENT" >&2; echo -n "#"' ' >&2; [ -n "$1" ] && echo -n "$@" >&2; }
_,() { echo -n "$@" >&2; }
_.() { echo >&2; }
__() { echo -n "$INDENT" >&2; echo -n "# ${RCM_INDENT}" >&2; [ -n "$1" ] && echo "$@" >&2; }
___() { echo -n "$INDENT" >&2; echo -n "# ${RCM_INDENT}${RCM_INDENT}" >&2; [ -n "$1" ] && echo "$@" >&2 || echo -n  >&2; }
____() { echo >&2; [ -n "$RCM_DELAY" ] && sleep "$RCM_DELAY"; }

# Define variables and constants.
RCM_DELAY=${RCM_DELAY:=.5}; [ -n "$fast" ] && unset RCM_DELAY
RCM_INDENT='    '; [ "$(tput cols)" -le 80 ] && RCM_INDENT='  '

# Functions.
printVersion() {
    echo '0.11.35'
}
printHelp() {
    title RCM Drupal Setup
    _ 'Variation '; yellow Bundle; _.
    _ 'Version '; yellow `printVersion`; _.
    _.
    # Populate variable $users.
    users=`cut -d: -f1 /etc/passwd | while read line; do [ -d /home/$line ] && echo " ${line}"; done | tr $'\n' ','`
    nginx_user=
    conf_nginx=`command -v nginx > /dev/null && nginx -V 2>&1 | grep -o -P -- '--conf-path=\K(\S+)'`
    if [ -f "$conf_nginx" ];then
        nginx_user=`grep -o -P '^user\s+\K([^;]+)' "$conf_nginx"`
    fi
    [ -n "$nginx_user" ] && { nginx_user=" ${nginx_user},"; }
    [ -n "$users" ] && users=" Available values:${nginx_user}${users} or other."
    cat << EOF
Usage: rcm-drupal-setup-variation-bundle [options]

Options:
   --variation *
        Select the variation setup. Values available from command: rcm-drupal-setup-variation-bundle(eligible).
   --project-name *
        Set the project name as identifier.
        Allowed characters are a-z, 0-9, and underscore (_).
   --url
        The value can be public or private domain, or URL.
        Drupal automatically has address at http://<project>.drupal.localhost/.
        Example: \`example.org\`, \`example.org/path/to/drupal/\`, or \`https://sub.example.org:8080/\`.
        Special top level domain such us .local, .example, etc will pretend as private domain.
   --php-fpm-user
        Set the Unix user that used by PHP FPM.
        Default value is the user that used by web server (the common name is www-data).
        If the user does not exists, it will be autocreate as reguler user.${users}
   --php-fpm-config
        Additional PHP-FPM Configuration inside pool directory.
        Available value: [1], [2], [3], [4], [5], [6], [7], or other.
        [1]: pm=ondemand
        [2]: php_flag[display_errors]=on
        [3]: php_value[max_execution_time]=300
        [4]: php_admin_value[memory_limit]=256M
        [5]: php_admin_value[upload_max_filesize]=25M
        [6]: php_admin_value[post_max_size]=1024M
        [7]: php_admin_flag[log_errors]=on
        Multivalue.
   --no-drush-install ^
        If selected, installation will continue to the browser.
        If you are choose Drupal CMS instead Drupal Core, it is recommended to continue installation in the browser.

Other options (For expert only):
   --prefix
        Set prefix directory for project.
        Default to home directory of --php-fpm-user or /usr/local/share.
   --project-container
        Set the container directory for all projects.
        Available value: drupal-projects, drupal, public_html, or other.
        Default to drupal-projects.
   --project-parent-name
        Set the project parent name. The parent is not have to installed before.
        Value available from command: ls-drupal(), or other.
   --no-sites-default ^
        Prevent installing drupal inside directory sites/default.
        Drupal will install inside sites/[<project-parent-name>--]<project-name>.
   --no-auto-add-group ^
        By default, if Nginx User cannot access PHP-FPM's Directory, auto add group of PHP-FPM User to Nginx User.
        Use this flag to omit that default action.
   --offline ^
        Set COMPOSER_DISABLE_NETWORK=1 to composer.
        Use this if you want to repeat the installation of a project with different name that has been successfully installed before.
   --without-certbot-obtain ^
        The dafault value is \`--with-certbot-obtain\`, it will check the value of
        \`--url\`. The URL that contains https scheme is will automatically obtain.
        The URL that not contains http or https, it means using https excepts special top level domain such us .local, .example, etc.
        Use this option to force certbot to use existing certificate if the \`--url\`
        contains https.
   --certificate-name
        Use the existing certificate name that issued by Let's encrypt or set a
        new name of certificate that to be obtained.

Global Options.
   --fast
        No delay every subtask.
   --version
        Print version of this script.
   --help
        Show this help.

Dependency:
   rcm-php-apt
   rcm-php-setup-adjust-cli-version
   rcm-php-setup-drupal:`printVersion`
   rcm-wsl-setup-lemp-stack
   rcm-composer-autoinstaller
   rcm-drupal-autoinstaller-nginx:`printVersion`
   rcm-drupal-setup-drush-alias:`printVersion`
   rcm-drupal-setup-internal-command-cd-drupal:`printVersion`
   rcm-drupal-setup-internal-command-ls-drupal:`printVersion`
   rcm-drupal-setup-internal-command-find-drupal:`printVersion`
   rcm-drupal-setup-dump-variables:`printVersion`
   rcm-drupal-setup-wrapper-nginx-virtual-host-autocreate-php-multiple-root:`printVersion`
   rcm-php-fpm-setup-project-config
   rcm-certbot-apt
   rcm-dig-watch-domain-exists

Download:
   [rcm-php-setup-drupal](https://github.com/ijortengab/drupal-autoinstaller/raw/master/rcm/php/rcm-php-setup-drupal.sh)
   [rcm-drupal-autoinstaller-nginx](https://github.com/ijortengab/drupal-autoinstaller/raw/master/rcm/drupal/rcm-drupal-autoinstaller-nginx.sh)
   [rcm-drupal-setup-drush-alias](https://github.com/ijortengab/drupal-autoinstaller/raw/master/rcm/drupal/rcm-drupal-setup-drush-alias.sh)
   [rcm-drupal-setup-internal-command-cd-drupal](https://github.com/ijortengab/drupal-autoinstaller/raw/master/rcm/drupal/rcm-drupal-setup-internal-command-cd-drupal.sh)
   [rcm-drupal-setup-internal-command-ls-drupal](https://github.com/ijortengab/drupal-autoinstaller/raw/master/rcm/drupal/rcm-drupal-setup-internal-command-ls-drupal.sh)
   [rcm-drupal-setup-internal-command-find-drupal](https://github.com/ijortengab/drupal-autoinstaller/raw/master/rcm/drupal/rcm-drupal-setup-internal-command-find-drupal.sh)
   [rcm-drupal-setup-dump-variables](https://github.com/ijortengab/drupal-autoinstaller/raw/master/rcm/drupal/rcm-drupal-setup-dump-variables.sh)
   [rcm-drupal-setup-wrapper-nginx-virtual-host-autocreate-php-multiple-root](https://github.com/ijortengab/drupal-autoinstaller/raw/master/rcm/drupal/rcm-drupal-setup-wrapper-nginx-virtual-host-autocreate-php-multiple-root.sh)
EOF
}

# Help and Version.
[ -n "$help" ] && { printHelp; exit 1; }
[ -n "$version" ] && { printVersion; exit 1; }

# Command.
command="$1"; shift
if [ -n "$command" ];then
    case "$command" in
        eligible) ;;
        *)
            # Bring back command as argument position.
            set -- "$command" "$@"
            # Reset command.
            command=
    esac
fi

# Functions.
eligible() {
    # chapter Available:
    eligible=()
    if [ -f /etc/os-release ];then
        . /etc/os-release
    fi
    _; _.
    longest_text='        #         Variation debian12e. Debian 12,    PHP 8.3, Drupal CMS 1.'
    if [[ $(tput cols) -gt "${#longest_text}" ]];then
        ___; _, 'Variation '; [[ "$ID" == debian && "$VERSION_ID" == 11 ]]    && color=green2 || color=red; $color debian11a; _, . Debian' '; hN 11; _, , '   'PHP' '; hN 8.1; _, , Drupal' '    ; hN 9.  ; _.; eligible+=("debian11a;debian;11")
        ___; _, 'Variation '; [[ "$ID" == debian && "$VERSION_ID" == 11 ]]    && color=green2 || color=red; $color debian11b; _, . Debian' '; hN 11; _, , '   'PHP' '; hN 8.2; _, , Drupal' '    ; hN 10. ; _.; eligible+=("debian11b;debian;11")
        ___; _, 'Variation '; [[ "$ID" == debian && "$VERSION_ID" == 11 ]]    && color=green2 || color=red; $color debian11c; _, . Debian' '; hN 11; _, , '   'PHP' '; hN 8.3; _, , Drupal' '    ; hN 10. ; _.; eligible+=("debian11c;debian;11")
        ___; _, 'Variation '; [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; $color ubuntu22a; _, . Ubuntu' '; hN 22.04;   _, , PHP' '; hN 8.1; _, , Drupal' '    ; hN 9.  ; _.; eligible+=("ubuntu22a;ubuntu;22.04")
        ___; _, 'Variation '; [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; $color ubuntu22b; _, . Ubuntu' '; hN 22.04;   _, , PHP' '; hN 8.2; _, , Drupal' '    ; hN 10. ; _.; eligible+=("ubuntu22b;ubuntu;22.04")
        ___; _, 'Variation '; [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; $color ubuntu22c; _, . Ubuntu' '; hN 22.04;   _, , PHP' '; hN 8.3; _, , Drupal' '    ; hN 10. ; _.; eligible+=("ubuntu22c;ubuntu;22.04")
        ___; _, 'Variation '; [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; $color ubuntu22d; _, . Ubuntu' '; hN 22.04;   _, , PHP' '; hN 8.3; _, , Drupal' '    ; hN 11. ; _.; eligible+=("ubuntu22d;ubuntu;22.04")
        ___; _, 'Variation '; [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; $color ubuntu22e; _, . Ubuntu' '; hN 22.04;   _, , PHP' '; hN 8.3; _, , Drupal CMS' '; hN 1.  ; _.; eligible+=("ubuntu22e;ubuntu;22.04")
        ___; _, 'Variation '; [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; $color debian12a; _, . Debian' '; hN 12; _, , '   'PHP' '; hN 8.1; _, , Drupal' '    ; hN 9.  ; _.; eligible+=("debian12a;debian;12")
        ___; _, 'Variation '; [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; $color debian12b; _, . Debian' '; hN 12; _, , '   'PHP' '; hN 8.2; _, , Drupal' '    ; hN 10. ; _.; eligible+=("debian12b;debian;12")
        ___; _, 'Variation '; [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; $color debian12c; _, . Debian' '; hN 12; _, , '   'PHP' '; hN 8.3; _, , Drupal' '    ; hN 10. ; _.; eligible+=("debian12c;debian;12")
        ___; _, 'Variation '; [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; $color debian12d; _, . Debian' '; hN 12; _, , '   'PHP' '; hN 8.3; _, , Drupal' '    ; hN 11. ; _.; eligible+=("debian12d;debian;12")
        ___; _, 'Variation '; [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; $color debian12e; _, . Debian' '; hN 12; _, , '   'PHP' '; hN 8.3; _, , Drupal CMS' '; hN 1.  ; _.; eligible+=("debian12e;debian;12")
        ___; _, 'Variation '; [[ "$ID" == ubuntu && "$VERSION_ID" == 24.04 ]] && color=green2 || color=red; $color ubuntu24a; _, . Ubuntu' '; hN 24.04;   _, , PHP' '; hN 8.3; _, , Drupal' '    ; hN 11. ; _.; eligible+=("ubuntu24a;ubuntu;24.04")
        ___; _, 'Variation '; [[ "$ID" == ubuntu && "$VERSION_ID" == 24.04 ]] && color=green2 || color=red; $color ubuntu24b; _, . Ubuntu' '; hN 24.04;   _, , PHP' '; hN 8.3; _, , Drupal CMS' '; hN 1.  ; _.; eligible+=("ubuntu24b;ubuntu;24.04")
    else
        tab_stop_position=( '21' '14' '9' )
        lines=()
        [[ "$ID" == debian && "$VERSION_ID" == 11 ]]    && color=green2 || color=red; eligible+=("debian11a;debian;11")
        lines+=("Variation <${color}>debian11a</${color}>. -->Debian <hN>11</hN>, -->PHP <hN>8.1</hN>, -->Drupal <hN>9</hN>.")
        [[ "$ID" == debian && "$VERSION_ID" == 11 ]]    && color=green2 || color=red; eligible+=("debian11b;debian;11")
        lines+=("Variation <${color}>debian11b</${color}>. -->Debian <hN>11</hN>, -->PHP <hN>8.2</hN>, -->Drupal <hN>10</hN>.")
        [[ "$ID" == debian && "$VERSION_ID" == 11 ]]    && color=green2 || color=red; eligible+=("debian11c;debian;11")
        lines+=("Variation <${color}>debian11c</${color}>. -->Debian <hN>11</hN>, -->PHP <hN>8.3</hN>, -->Drupal <hN>10</hN>.")
        [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; eligible+=("ubuntu22a;ubuntu;22.04")
        lines+=("Variation <${color}>ubuntu22a</${color}>. -->Ubuntu <hN>22.04</hN>, -->PHP <hN>8.1</hN>, -->Drupal <hN>9</hN>.")
        [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; eligible+=("ubuntu22b;ubuntu;22.04")
        lines+=("Variation <${color}>ubuntu22b</${color}>. -->Ubuntu <hN>22.04</hN>, -->PHP <hN>8.2</hN>, -->Drupal <hN>10</hN>.")
        [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; eligible+=("ubuntu22c;ubuntu;22.04")
        lines+=("Variation <${color}>ubuntu22c</${color}>. -->Ubuntu <hN>22.04</hN>, -->PHP <hN>8.3</hN>, -->Drupal <hN>10</hN>.")
        [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; eligible+=("ubuntu22d;ubuntu;22.04")
        lines+=("Variation <${color}>ubuntu22d</${color}>. -->Ubuntu <hN>22.04</hN>, -->PHP <hN>8.3</hN>, -->Drupal <hN>11</hN>.")
        [[ "$ID" == ubuntu && "$VERSION_ID" == 22.04 ]] && color=green2 || color=red; eligible+=("ubuntu22e;ubuntu;22.04")
        lines+=("Variation <${color}>ubuntu22e</${color}>. -->Ubuntu <hN>22.04</hN>, -->PHP <hN>8.3</hN>, -->Drupal CMS <hN>1</hN>.")
        [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; eligible+=("debian12a;debian;12")
        lines+=("Variation <${color}>debian12a</${color}>. -->Debian <hN>12</hN>, -->PHP <hN>8.1</hN>, -->Drupal <hN>9</hN>.")
        [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; eligible+=("debian12b;debian;12")
        lines+=("Variation <${color}>debian12b</${color}>. -->Debian <hN>12</hN>, -->PHP <hN>8.2</hN>, -->Drupal <hN>10</hN>.")
        [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; eligible+=("debian12c;debian;12")
        lines+=("Variation <${color}>debian12c</${color}>. -->Debian <hN>12</hN>, -->PHP <hN>8.3</hN>, -->Drupal <hN>10</hN>.")
        [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; eligible+=("debian12d;debian;12")
        lines+=("Variation <${color}>debian12d</${color}>. -->Debian <hN>12</hN>, -->PHP <hN>8.3</hN>, -->Drupal <hN>11</hN>.")
        [[ "$ID" == debian && "$VERSION_ID" == 12 ]]    && color=green2 || color=red; eligible+=("debian12e;debian;12")
        lines+=("Variation <${color}>debian12e</${color}>. -->Debian <hN>12</hN>, -->PHP <hN>8.3</hN>, -->Drupal CMS <hN>1</hN>.")
        [[ "$ID" == ubuntu && "$VERSION_ID" == 24.04 ]] && color=green2 || color=red; eligible+=("ubuntu24a;ubuntu;24.04")
        lines+=("Variation <${color}>ubuntu24a</${color}>. -->Ubuntu <hN>24.04, -->PHP <hN>8.3</hN>, -->Drupal <hN>11</hN>.")
        [[ "$ID" == ubuntu && "$VERSION_ID" == 24.04 ]] && color=green2 || color=red; eligible+=("ubuntu24b;ubuntu;24.04")
        lines+=("Variation <${color}>ubuntu24b</${color}>. -->Ubuntu <hN>24.04, -->PHP <hN>8.3</hN>, -->Drupal CMS <hN>1</hN>.")
        lines_cloned=("${lines[@]}")
        lines=()
        for line in "${lines_cloned[@]}"; do
            lines+=("$(echo "$line" | sed -E s,-+\>,$'\t',g)")
        done
        unset lines_cloned

        # Output di print per baris, secara UX kelamaan.
        # wordWrapParagraph lines[@] --indent=2 --indent-hanging=1

        # Output di print semua sekaligus, secara UX kelamaan juga.
        # tempfile=$(mktemp -p /dev/shm -t rcm-drupal.XXXXXX)
        # wordWrapParagraph lines[@] --indent=2 --indent-hanging=1 --output="$tempfile" &
        # pid=$!
        # spin='-\|/'
        # i=0
        # while kill -0 $pid 2>/dev/null
        # do
          # i=$(( (i+1) %4 ))
          # printf "\r" >&2; __; _, "Waiting...${spin:$i:1}"
          # sleep .1
        # done
        # printf "\r\033[K" >&2;
        # cat "$tempfile" >&2

        # Output di print per 2 baris, secara UX mendingan.
        # Syaratnya diwajibakan set global variable tab_stop_position
        lines_count=${#lines[@]}
        lines_cloned=("${lines[@]}")
        lines=()
        lines_last_index=$((lines_count-1))
        for ((x = 0 ; x < $lines_count ; x++)); do
            let y++
            lines+=("${lines_cloned[$x]}")
            test=$(( ${#lines[@]}%2))
            if [[ $test -eq 0  || $x -eq $lines_last_index ]];then
                [ -n "$tempfile" ] || tempfile=$(mktemp -p /dev/shm -t rcm-drupal.XXXXXX)

                wordWrapParagraph lines[@] --indent=2 --indent-hanging=1 --output="$tempfile" &
                pid=$!
                spin='-\|/'
                while kill -0 $pid 2>/dev/null
                do
                  i=$(( (i+1) %4 ))
                  printf "\r" >&2; __; _, "Waiting...${spin:$i:1}"
                  sleep .1
                done
                printf "\r\033[K" >&2;
                cat "$tempfile" >&2

                :> "$tempfile"
                unset lines
                lines=()
            fi
        done
        [ -n "$tempfile" ] && rm "$tempfile"
    fi

    for each in "${eligible[@]}";do
        variation=$(cut -d';' -f1 <<< "$each")
        _id=$(cut -d';' -f2 <<< "$each")
        _version_id=$(cut -d';' -f3 <<< "$each")
        if [[ "$_id" == "$ID" && "$_version_id" == "$VERSION_ID" ]];then
            echo $variation
        fi
    done
}
green2() {
    local word=$1
    hN "$word" green
}
hN() {
    # hightlightNumber
    local other=$2 i
    [ -z "$other" ] && other=_,
    local number=yellow
    local word=$1 segment
    local current last
    for ((i = 0 ; i < ${#word} ; i++)); do
        if [[ ${word:$i:1} =~ ^[0-9]+$ ]];then
            current=number
        else
            current=other
        fi
        if [[ -n "$last" && ! "$last" == "$current" ]];then
            ${!last} $segment
            segment=
        fi
        last="$current"
        segment+=${word:$i:1}
    done
    ${!last} $segment
}
explodeParagraphs() {
    # Explode kalimat by space, tanpa menghilangkan space itu sendiri.
    # Karena space diperlukan untuk menghitung panjang karakter sehingga
    # tidak bisa dihapuskan.
    # Space berada pada prefix each word.
    # Trailing space pada kalimat menjadi milik last word.
    #
    # global words_array
    byTab() {
        # Explode lagi berdasarkan karakter TAB.
        # global words_array
        local words_array_clone each tab left_tab right_tab string
        words_array_clone=("${words_array[@]}")
        words_array=()
        for each in "${words_array_clone[@]}"; do
            string="$each"
            if [[ "$string" =~ $'\t' ]];then
                while IFS= read line; do
                    tab="$line"
                    left_tab="${string%%${tab}*}"
                    [ -n "$left_tab" ] && words_array+=("$left_tab")
                    words_array+=("$tab")
                    right_tab="${string#${left_tab}}"
                    right_tab=${right_tab:1}
                    string="$right_tab"
                done <<< `echo "$string" | grep -E -o $'\t'`
                [ -n "$string" ] && words_array+=("$string")
            else
                [ -n "$string" ] && words_array+=("$string")
            fi
        done
    }
    byTag() {
        # Explode lagi berdasarkan tag <string>.
        # global words_array
        local words_array_clone each tag left_tag right_tag string
        words_array_clone=("${words_array[@]}")
        words_array=()
        for each in "${words_array_clone[@]}"; do
            string="$each"
            # if grep -q -E '</?[^</>]+>' <<< "$string";then
            if [[ "$string" =~ \</?[^\</\>]+\> ]];then
                while IFS= read line; do
                    tag="$line"
                    left_tag="${string%%${tag}*}"
                    [ -n "$left_tag" ] && words_array+=("$left_tag")
                    words_array+=("$tag")
                    right_tag="${string#${left_tag}}"
                    right_tag=${right_tag:${#tag}}
                    string="$right_tag"
                done <<< `echo "$string" | grep -E -o '</?[^</>]+>'`
                [ -n "$string" ] && words_array+=("$string")
            else
                [ -n "$string" ] && words_array+=("$string")
            fi
        done
    }
    bySpace() {
        # Explode lagi berdasarkan space.
        # global words_array
        local words_array_clone each tag left_tag right_tag string
        words_array_clone=("${words_array[@]}")
        words_array=()
        for each in "${words_array_clone[@]}"; do
            string="$each"
            if [[ "$string" =~ ' ' ]];then
                while IFS= read line; do
                    tag="$line"
                    left_tag="${string%%${tag}*}"
                    [ -n "$left_tag" ] && words_array+=("$left_tag")
                    words_array+=("$tag")
                    right_tag="${string#${left_tag}}"
                    right_tag=${right_tag:${#tag}}
                    string="$right_tag"
                done <<< `echo "$string" | grep -E -o ' +'`
                [ -n "$string" ] && words_array+=("$string")
            else
                [ -n "$string" ] && words_array+=("$string")
            fi
        done
    }
    local string="$1"; shift
    # Jika karakter spasi semuanya, maka:
    if [[ "$string" =~ ^[\ ]+$ ]];then
        words_array=("$string")
        return
    fi
    local by_tab by_tag by_space
    while [[ $# -gt 0 ]]; do
        case "$1" in
            +byTab) by_tab=1; shift ;;
            +byTag) by_tag=1; shift ;;
            +bySpace) by_space=1; shift ;;
            *) shift ;;
        esac
    done
    local line leading_space trailing_space
    words_array=()
    leading_space=$(echo "$string" | grep -E -o '^[\ ]+')
    trailing_space=$(echo "$string" | grep -E -o '[\ ]+$')
    if [ -n "$leading_space" ];then
        string=$(echo "$string" | sed -E 's|^[\ ]+(.*)|\1|')
    fi
    if [ -n "$trailing_space" ];then
        string=$(echo "$string" | sed -E 's|(.*)[\ ]+$|\1|')
    fi
    words_array[0]=$(echo "$string" | grep -E -o '^[^\ ]+')
    words_array[0]="$leading_space""${words_array[0]}"
    local i=1
    while IFS= read line; do
        if [ -n "$line" ];then
            words_array[$i]="$line"
            let i++
        fi
    done <<< `echo "$string" | grep -E -o '[\ ]+[^\ ]+'`
    if [ $i -gt 0 ];then
        let i--
        words_array[$i]="${words_array[$i]}""$trailing_space"
    fi
    [ -n "$by_tab" ] && byTab
    [ -n "$by_tag" ] && byTag
    [ -n "$by_space" ] && bySpace
}
wordWrapParagraph() {
    cleaningTag() {
        # Menghilangkan tag warna, open tag maupun close tag.
        # global each
        # global color
        local string="$1"
        opentag=$(echo "$string" | grep -E -o '<[^</>]+>')
        if [ -n "$opentag" ];then
            # color=$(echo "$opentag" | grep -E -o '[^<>]+')
            color="${opentag:1:-1}"
            string=${string//"$opentag"/}
            closetag="</${color}>"
            # if grep -q -F "$closetag" <<< "$string";then
            if [[ "$string" =~ "$closetag" ]];then
                color_stop=1
                string=${string//"$closetag"/}
            fi
        fi
        each="$string"
    }
    cleaningCloseTag() {
        # global each
        # global color
        local string="$1"
        closetag="</${color}>"
        # if grep -q -F "$closetag" <<< "$string";then
        if [[ "$string" =~ "$closetag" ]];then
            color_stop=1
            string=${string//"$closetag"/}
        fi
        each="$string"
    }
    colorStop() {
        # global color_stop
        # global default_color
        if [ -n "$color_stop" ];then
            color=$default_color
            color_stop=
        fi
    }
    strip_tags() {
        # global _return
        local string="$1" each
        _return=
        if [[ "$string" =~ \</?[^\</\>]+\> ]];then
            while IFS= read line; do
                tag="$line"
                left_tag="${string%%${tag}*}"
                [ -n "$left_tag" ] && _return+="$left_tag"
                right_tag="${string#${left_tag}}"
                right_tag=${right_tag:${#tag}}
                string="$right_tag"
            done <<< `echo "$string" | grep -E -o '</?[^</>]+>'`
            [ -n "$string" ] && _return+="$string"
        else
            [ -n "$string" ] && _return+="$string"
        fi
    }
    calculateTabStopPosition() {
        # global tab_stop_position
        local string="$1" words_array each left_tab tab _return
        tab=$'\t'
        left_tab="${string%%${tab}*}"
        local i=0
        until [[ ! "${#left_tab}" -lt "${#string}" ]];do
            right_tab="${string#${left_tab}}"
            right_tab=${right_tab:1}
            # Populate variable $_return.
            strip_tags "$left_tab"
            _current="${tab_stop_position[$i]}"
            if [ -n "$_current" ];then
                if [ "${#_return}" -gt "$_current" ];then
                    tab_stop_position[$i]="${#_return}"
                fi
            else
                tab_stop_position[$i]="${#_return}"
            fi
            string="$right_tab"
            left_tab="${string%%${tab}*}"
            let i++
        done
    }
    wordWrapSentence() {
        populateHangingIndent() {
            # global each
            # global hanging_indent_additional
            # global indent_hanging
            # global tab_stop_position
            hanging_indent_additional=0
            if [[ "$indent_hanging" -gt 0 && "${#tab_stop_position[@]}" -gt 0 ]];then
                for ((tab_index = 0 ; tab_index < $indent_hanging ; tab_index++)); do
                    hanging_indent_additional=$((hanging_indent_additional + ${tab_stop_position[$tab_index]}))
                done
            fi
            hanging_indent_additional=$(printf %"${hanging_indent_additional}"s)
            # Trim leading space.
            each=$(echo "$each" | sed -E 's|^[\ ]+(.*)|\1|')
            if [[ "$each" =~ ^[\ ]+$ ]];then
                each=
            fi
        }
        colorize() {
            local string="$1" words_array color each last opentag closetag
            if [[ ! "$string" =~ [\<\>] ]];then
                # Tanpa warna. Contoh: "Variation"
                _, "$string"
            elif [[ "$string" =~ ^\<[^\<\>]+\>[^\<\>]+\</[^\<\>]+\>[\.,]?$ ]];then
                # Hanya satu warna.
                # Contoh: "<red>debian11a</red>."
                #         "<hN>11</hN>,"
                #         "<hN>8.1</hN>,"
                color="${string#<}"
                color="${color/%>*}"
                last="${string/#*>}"
                opentag="<${color}>"
                closetag="</${color}>"
                string=${string//"$opentag"/}
                string=${string//"${closetag}${last}"/}
                $color "$string"
                [ -n "$last" ] && _, "$last"
            else
                # Banyak warna.
                # Contoh: "<green2>debian12apakah76</green2>apakahyaa<red>tempe</red>.."
                # _, "$string"
                [ -z $default_color ] && default_color=_,
                color="$default_color"
                explodeParagraphs "$string" +byTag
                for each in "${words_array[@]}"; do
                    cleaningTag "$each"
                    cleaningCloseTag "$each"
                    $color "$each"
                    colorStop
                done
            fi
        }
        # global cols
        # global indent_first_line
        # global indent_hanging
        # global tab_stop_position
        local paragraph="$1" words_array default_color="$2" color
        local each each_original each_stripped
        local current_line _current_line first_line last hanging_indent_additional
        local tab_index=0
        local max=0
        local min=0
        max=$cols
        local _indent_first_line
        _indent_first_line=$((indent_first_line + 2))
        # Angka 2 adalah tambahan dari '# '.
        _max=$((100 + ${#INDENT} + $_indent_first_line))
        if [ $max -gt $_max ];then
            max=100
            min=80
        else
            max=$((max - ${#INDENT} - $_indent_first_line))
            min="$max"
        fi
        explodeParagraphs "$paragraph" +byTab +byTag +bySpace
        current_line=
        merged=
        local words_array_clone=("${words_array[@]}")
        words_array=()
        for each in "${words_array_clone[@]}"; do
            if [[ ! "$each" =~ ^[\ $'\t']+$ ]];then
                merged+="$each"
            else
                if [ -n "$merged" ];then
                    words_array+=("$merged")
                    merged=
                fi
                words_array+=("$each")
            fi
        done
        if [ -n "$merged" ];then
            words_array+=("$merged")
        fi
        unset words_array_clone
        local i=0
        local count="${#words_array[@]}"
        first_line=1
        for each in "${words_array[@]}"; do
            each_original="$each"
            if [[ "$each" == $'\t' ]];then
                each=
                local x
                next_tab=0
                for ((x = 0 ; x <= $tab_index ; x++)); do
                    next_tab=$((next_tab + ${tab_stop_position[$x]}))
                done
                if [[ "$next_tab" -gt "${#current_line}" ]];then
                    additional_space=$((next_tab - "${#current_line}" ))
                    each=$(printf %"${additional_space}"s)
                fi
                let tab_index++
            fi
            strip_tags "$each"
            each_stripped="$_return"
            let i++
            [ "$i" == "$count" ] && last=1 || last=
            if [ -z "$current_line" ]; then
                if [ -n "$first_line" ];then
                    first_line=
                    current_line="$each_stripped"
                    _; printf %"${indent_first_line}"s >&2; colorize "$each"
                else
                    current_line="$each_stripped"
                    _; printf %"${indent_first_line}"s >&2;
                    populateHangingIndent
                    current_line="${hanging_indent_additional}${each_stripped}"
                    _, "$hanging_indent_additional"; colorize "$each"
                fi
                if [ -n "$last" ];then
                    _.
                fi
            else
                _current_line="${current_line}${each_stripped}"
                if [ "${#_current_line}" -gt $max ];then
                    if [[ "$each_original" == $'\t' &&  "$each_stripped" =~ ^[\ ]+$ ]];then
                        each_stripped=
                        each=
                        _current_line="${current_line}${each_stripped}"
                    fi
                fi
                if [ "${#_current_line}" -le $min ];then
                    current_line+="$each_stripped"
                    colorize "$each"
                    if [ -n "$last" ];then
                        _.
                    fi
                elif [ "${#_current_line}" -le $max ];then
                    colorize "$each"; _.
                    current_line=
                else
                    _.;
                    _; printf %"${indent_first_line}"s >&2;
                    populateHangingIndent
                    current_line="${hanging_indent_additional}${each_stripped}"
                    _, "$hanging_indent_additional"; colorize "$each"
                    if [ -n "$last" ];then
                        _.
                    fi
                fi
            fi
        done
    }
    # global tab_stop_position
    local lines=("${!1}"); shift
    local line
    local output indent_first_line indent_hanging tempfile
    while [[ $# -gt 0 ]]; do
        case "$1" in
            --indent=*) indent_first_line="${1#*=}"; shift ;;
            --indent-hanging=*) indent_hanging="${1#*=}"; shift ;;
            --output=*) output="${1#*=}"; shift ;;
            *) shift ;;
        esac
    done
    local cols=$(tput cols)
    indent_first_line=$((indent_first_line*${#RCM_INDENT}))
    [ -z "$indent_hanging" ] && indent_hanging=0
    # Hitung karakter [tab] di setiap baris.
    # Boleh predefined untuk mempercepat karena variable tab_stop_position
    # adalah variable global.
    if [[ "${#tab_stop_position[@]}" -eq 0 ]];then
        for line in "${lines[@]}"; do
            calculateTabStopPosition "$line"
        done
    fi
    # echo '"${tab_stop_position[@]}"' "${tab_stop_position[@]}" >&2
    # Start drawing.
    if [ -n "$output" ];then
        tempfile="$output"
        for line in "${lines[@]}"; do
            wordWrapSentence "$line" 2>> "$tempfile"
        done
    else
        temp=
        for line in "${lines[@]}"; do
            temp=$(wordWrapSentence "$line" 2>&1)
            echo "$temp" >&2
        done
    fi
}

# Execute command.
if [[ -n "$command" && $(type -t "$command") == function ]];then
    "$command"
    exit 0
fi

# Title.
title rcm-drupal-setup-variation-bundle
____

[ "$EUID" -ne 0 ] && { error This script needs to be run with superuser privileges.; x; }

# Dependency.
while IFS= read -r line; do
    [[ -z "$line" ]] || command -v `cut -d: -f1 <<< "${line}"` >/dev/null || { error Unable to proceed, command not found: '`'`cut -d: -f1 <<< "${line}"`'`'.; x; }
done <<< `printHelp 2>/dev/null | sed -n '/^Dependency:/,$p' | sed -n '2,/^\s*$/p' | sed 's/^ *//g'`

#  Functions.
validateMachineName() {
    local value="$1" _value
    local parameter="$2"
    if [[ $value = *" "* ]];then
        [ -n "$parameter" ]  && error "Variable $parameter can not contain space."
        return 1;
    fi
    _value=$(sed -E 's|[^a-zA-Z0-9]|_|g' <<< "$value" | sed -E 's|_+|_|g' )
    if [[ ! "$value" == "$_value" ]];then
        error "Variable $parameter can only contain alphanumeric and underscores."
        _ 'Suggest: '; yellow "$_value"; _.
        return 1
    fi
}
Rcm_parse_url() {
    # Reset
    PHP_URL_SCHEME=
    PHP_URL_HOST=
    PHP_URL_PORT=
    PHP_URL_USER=
    PHP_URL_PASS=
    PHP_URL_PATH=
    PHP_URL_QUERY=
    PHP_URL_FRAGMENT=
    PHP_URL_SCHEME="$(echo "$1" | grep :// | sed -e's,^\(.*\)://.*,\1,g')"
    _PHP_URL_SCHEME_SLASH="${PHP_URL_SCHEME}://"
    _PHP_URL_SCHEME_REVERSE="$(echo ${1/${_PHP_URL_SCHEME_SLASH}/})"
    if grep -q '#' <<< "$_PHP_URL_SCHEME_REVERSE";then
        PHP_URL_FRAGMENT=$(echo $_PHP_URL_SCHEME_REVERSE | cut -d# -f2)
        _PHP_URL_SCHEME_REVERSE=$(echo $_PHP_URL_SCHEME_REVERSE | cut -d# -f1)
    fi
    if grep -q '\?' <<< "$_PHP_URL_SCHEME_REVERSE";then
        PHP_URL_QUERY=$(echo $_PHP_URL_SCHEME_REVERSE | cut -d? -f2)
        _PHP_URL_SCHEME_REVERSE=$(echo $_PHP_URL_SCHEME_REVERSE | cut -d? -f1)
    fi
    _PHP_URL_USER_PASS="$(echo $_PHP_URL_SCHEME_REVERSE | grep @ | cut -d@ -f1)"
    PHP_URL_PASS=`echo $_PHP_URL_USER_PASS | grep : | cut -d: -f2`
    if [ -n "$PHP_URL_PASS" ]; then
        PHP_URL_USER=`echo $_PHP_URL_USER_PASS | grep : | cut -d: -f1`
    else
        PHP_URL_USER=$_PHP_URL_USER_PASS
    fi
    _PHP_URL_HOST_PORT="$(echo ${_PHP_URL_SCHEME_REVERSE/$_PHP_URL_USER_PASS@/} | cut -d/ -f1)"
    PHP_URL_HOST="$(echo $_PHP_URL_HOST_PORT | sed -e 's,:.*,,g')"
    if grep -q -E ':[0-9]+$' <<< "$_PHP_URL_HOST_PORT";then
        PHP_URL_PORT="$(echo $_PHP_URL_HOST_PORT | sed -e 's,^.*:,:,g' -e 's,.*:\([0-9]*\).*,\1,g' -e 's,[^0-9],,g')"
    fi
    _PHP_URL_HOST_PORT_LENGTH=${#_PHP_URL_HOST_PORT}
    _LENGTH="$_PHP_URL_HOST_PORT_LENGTH"
    if [ -n "$_PHP_URL_USER_PASS" ];then
        _PHP_URL_USER_PASS_LENGTH=${#_PHP_URL_USER_PASS}
        _LENGTH=$((_LENGTH + 1 + _PHP_URL_USER_PASS_LENGTH))
    fi
    PHP_URL_PATH="${_PHP_URL_SCHEME_REVERSE:$_LENGTH}"

    # Debug
    # e '"$PHP_URL_SCHEME"' "$PHP_URL_SCHEME"; _.
    # e '"$PHP_URL_HOST"' "$PHP_URL_HOST"; _.
    # e '"$PHP_URL_PORT"' "$PHP_URL_PORT"; _.
    # e '"$PHP_URL_USER"' "$PHP_URL_USER"; _.
    # e '"$PHP_URL_PASS"' "$PHP_URL_PASS"; _.
    # e '"$PHP_URL_PATH"' "$PHP_URL_PATH"; _.
    # e '"$PHP_URL_QUERY"' "$PHP_URL_QUERY"; _.
    # e '"$PHP_URL_FRAGMENT"' "$PHP_URL_FRAGMENT"; _.
}
fileMustExists() {
    # global used:
    # global modified:
    # function used: __, success, error, x
    if [ -f "$1" ];then
        __; green File '`'$(basename "$1")'`' ditemukan.; _.
    else
        __; red File '`'$(basename "$1")'`' tidak ditemukan.; x
    fi
}
resolve_relative_path() {
    if [ -d "$1" ];then
        cd "$1" || return 1
        pwd
    elif [ -e "$1" ];then
        if [ ! "${1%/*}" = "$1" ]; then
            cd "${1%/*}" || return 1
        fi
        echo "$(pwd)/${1##*/}"
    else
        return 1
    fi
}
ArraySearch() {
    local index match="$1"
    local source=("${!2}")
    for index in "${!source[@]}"; do
       if [[ "${source[$index]}" == "${match}" ]]; then
           _return=$index; return 0
       fi
    done
    return 1
}

# Requirement, validate, and populate value.
chapter Dump variable.
[ -n "$fast" ] && isfast=' --fast' || isfast=''
PREFIX_MASTER=${PREFIX_MASTER:=/usr/local/share/drupal}
code 'PREFIX_MASTER="'$PREFIX_MASTER'"'
PROJECTS_CONTAINER_MASTER=${PROJECTS_CONTAINER_MASTER:=projects}
code 'PROJECTS_CONTAINER_MASTER="'$PROJECTS_CONTAINER_MASTER'"'
code no_auto_add_group="$no_auto_add_group"
code 'no_sites_default="'$no_sites_default'"'
[ -n "$no_auto_add_group" ] && is_auto_add_group='' || is_auto_add_group=' --auto-add-group'
[ -n "$no_sites_default" ] && is_no_sites_default=' --no-sites-default' || is_no_sites_default=''
if [ -z "$variation" ];then
    error "Argument --variation required."; x
fi
case "$variation" in
    debian11b) os=debian; os_version=11   ; php_version=8.2; drupal_version=10   ;;
    debian11a) os=debian; os_version=11   ; php_version=8.1; drupal_version=9    ;;
    ubuntu22b) os=ubuntu; os_version=22.04; php_version=8.2; drupal_version=10   ;;
    ubuntu22a) os=ubuntu; os_version=22.04; php_version=8.1; drupal_version=9    ;;
    debian12b) os=debian; os_version=12   ; php_version=8.2; drupal_version=10   ;;
    debian12a) os=debian; os_version=12   ; php_version=8.1; drupal_version=9    ;;
    debian12c) os=debian; os_version=12   ; php_version=8.3; drupal_version=10   ;;
    debian11c) os=debian; os_version=11   ; php_version=8.3; drupal_version=10   ;;
    ubuntu22c) os=ubuntu; os_version=22.04; php_version=8.3; drupal_version=10   ;;
    ubuntu22d) os=ubuntu; os_version=22.04; php_version=8.3; drupal_version=11   ;;
    debian12d) os=debian; os_version=12   ; php_version=8.3; drupal_version=11   ;;
    debian12e) os=debian; os_version=12   ; php_version=8.3; drupalcms_version=1 ;;
    ubuntu24a) os=ubuntu; os_version=24.04; php_version=8.3; drupal_version=11   ;;
    ubuntu22e) os=ubuntu; os_version=22.04; php_version=8.3; drupalcms_version=1 ;;
    ubuntu24b) os=ubuntu; os_version=24.04; php_version=8.3; drupalcms_version=1 ;;
    *) error "Argument --variation is not valid."; x;;
esac
code php_version="$php_version"
code drupal_version="$drupal_version"
if [ -z "$project_name" ];then
    error "Argument --project-name required."; x
fi
code 'project_name="'$project_name'"'
if ! validateMachineName "$project_name" project_name;then x; fi
code 'project_parent_name="'$project_parent_name'"'
if [ -n "$project_parent_name" ];then
    if ! validateMachineName "$project_parent_name" project_parent_name;then x; fi
fi
is_wsl=
if [ -f /proc/sys/kernel/osrelease ];then
    read osrelease </proc/sys/kernel/osrelease
    if [[ "$osrelease" =~ microsoft || "$osrelease" =~ Microsoft ]];then
        is_wsl=1
    fi
fi
code 'is_wsl="'$is_wsl'"'
code 'url="'$url'"'
tld_special=(example test onion invalid local localhost alt)
is_tld_special=
if [ -n "$url" ];then
    Rcm_parse_url "$url"
	if [ -z "$PHP_URL_HOST" ];then
        error Argument --url is not valid: '`'"$url"'`'.; x
    fi
    [ -n "$PHP_URL_SCHEME" ] && url_scheme="$PHP_URL_SCHEME" || url_scheme=https
    if [ -z "$PHP_URL_PORT" ];then
        case "$url_scheme" in
            http) url_port=80;;
            https) url_port=443;;
        esac
    else
        url_port="$PHP_URL_PORT"
    fi
    url_host="$PHP_URL_HOST"
    url_path="$PHP_URL_PATH"
    # Modify variable url, auto add scheme.
    url_path_clean_trailing=$(echo "$url_path" | sed -E 's|/+$||g')
    # Modify variable url, auto trim trailing slash, auto add port.
    tld="${url_host##*.}"
    if ArraySearch "$tld" tld_special[@];then
        url_scheme=http
        if [ -z "$PHP_URL_PORT" ];then
            url_port=80
        fi
        is_tld_special=1
    fi
    _url_port=
    if [ -n "$url_port" ];then
        if [[ "$url_scheme" == https && "$url_port" == 443 ]];then
            _url_port=
        elif [[ "$url_scheme" == http && "$url_port" == 80 ]];then
            _url_port=
        else
            _url_port=":${url_port}"
        fi
    fi
    url="${url_scheme}://${url_host}${_url_port}${url_path_clean_trailing}"
fi
code 'url="'$url'"'
if [ -n "$project_parent_name" ];then
    url_dirname_website_info="${PREFIX_MASTER}/${PROJECTS_CONTAINER_MASTER}/${project_parent_name}/subprojects/${project_name}"
else
    url_dirname_website_info="${PREFIX_MASTER}/${PROJECTS_CONTAINER_MASTER}/${project_name}"
fi
code 'url_dirname_website_info="'$url_dirname_website_info'"'
code 'prefix="'$prefix'"'
if [ -n "$prefix" ];then
    if [ -d "$prefix" ];then
        if [ ! "${prefix:0:1}" == / ];then
            prefix=$(resolve_relative_path "$prefix")
        fi
    else
        error Directory prefix is not exists; x
    fi
fi
code 'prefix="'$prefix'"'
code 'no_drush_install="'$no_drush_install'"'
[ -n "$no_drush_install" ] && is_drush_install='' || is_drush_install=' --drush-install'
code 'offline="'$offline'"'
[ -n "$offline" ] && is_composer_offline=' --composer-offline' || is_composer_offline=''
if [ -n "$is_wsl" ];then
    # Jika mesin menggunakan WSL2, maka tambahkan max_execution_time (waktu proses)
    php_fpm_config=(pm=ondemand php_value[max_execution_time]=60 "${php_fpm_config[@]}")
fi
is_config_line=
is_config_line_array=()
# Dump array dengan single quote.
e; magenta 'php_fpm_config=('
first=1
for each in "${php_fpm_config[@]}";do
    if [ -n "$first" ];then
        magenta "'""$each""'"; first=
    else
        magenta " '""$each""'";
    fi
    [[ "$each" =~ ' ' ]] && is_config_line+=" --config-line='${each}'" || is_config_line+=" --config-line=${each}"
    is_config_line_array+=("--config-line=${each}")
done
magenta ')'; _.
code 'certificate_name="'$certificate_name'"'
if [ -n "$project_parent_name" ];then
    php_fpm_section="${project_parent_name}__${project_name}__drupal"
else
    php_fpm_section="${project_name}__drupal"
fi
code 'php_fpm_section="'$php_fpm_section'"'
[ -z "$certbot_obtain" ] && certbot_obtain=1
[ "$certbot_obtain" == 0 ] && certbot_obtain=
code 'certbot_obtain="'$certbot_obtain'"'
if [ -n "$is_tld_special" ];then
    certbot_obtain=
fi
code 'certbot_obtain="'$certbot_obtain'"'
[ -n "$certbot_obtain" ] && is_certbot_obtain=' --with-certbot-obtain' || is_certbot_obtain=' --without-certbot-obtain'
____

INDENT+="    " \
rcm-php-apt $isfast \
    --php-version="$php_version" \
    && INDENT+="    " \
rcm-php-setup-adjust-cli-version $isfast \
    --php-version="$php_version" \
    && INDENT+="    " \
rcm-php-setup-drupal $isfast \
    --php-version="$php_version" \
    ; [ ! $? -eq 0 ] && x

if [[ -n "$url" && -n "$certbot_obtain" ]];then
    INDENT+="    " \
    rcm-dig-watch-domain-exists $isfast \
        --domain="$url_host" \
        --waiting-time="60" \
        ; [ ! $? -eq 0 ] && x
fi

# Di baris ini seharusnya sudah terinstall nginx.
chapter Populate variables.
nginx_user=
conf_nginx=`command -v nginx > /dev/null && command -v nginx > /dev/null && nginx -V 2>&1 | grep -o -P -- '--conf-path=\K(\S+)'`
if [ -f "$conf_nginx" ];then
    nginx_user=`grep -o -P '^user\s+\K([^;]+)' "$conf_nginx"`
fi
code 'nginx_user="'$nginx_user'"'
if [ -z "$nginx_user" ];then
    error "Variable \$nginx_user failed to populate."; x
fi
if [ -z "$php_fpm_user" ];then
    php_fpm_user="$nginx_user"
fi
code 'php_fpm_user="'$php_fpm_user'"'
____

if [ -n "$is_wsl" ];then
    INDENT+="    " \
    rcm-wsl-setup-lemp-stack $isfast \
        --php-version="$php_version" \
        ; [ ! $? -eq 0 ] && x
fi

# Contoh jika terdapat kasus sbb:
# --config-line='php_flag[display_errors] = on'
# Tidak bisa pakai cara dibawah ini jika terdapat karakter spasi.
# INDENT+="    " \
# rcm-php-fpm-setup-project-config $isfast \
#   $is_config_line \
# Namun bisa jika menggunakan array yang diwrap dengan double quote.
INDENT+="    " \
rcm-php-fpm-setup-project-config $isfast \
    "${is_config_line_array[@]}" \
    --php-version="$php_version" \
    --php-fpm-user="$php_fpm_user" \
    --section="$php_fpm_section" \
    ; [ ! $? -eq 0 ] && x

# Di baris ini seharusnya sudah exists user linux $php_fpm_user.
chapter Populate variables.
if [ -z "$prefix" ];then
    prefix=$(getent passwd "$php_fpm_user" | cut -d: -f6 )
fi
# Jika $php_fpm_user adalah nginx, maka $HOME nya adalah /nonexistent, maka
# perlu kita verifikasi lagi.
if [ ! -d "$prefix" ];then
    prefix=
fi
if [ -z "$prefix" ];then
    prefix=/usr/local/share
    project_container=drupal
fi
if [ -z "$project_container" ];then
    project_container=drupal-projects
fi
code 'prefix="'$prefix'"'
code 'project_container="'$project_container'"'
# Hanya populate, tidak harus exists direktori ini.
project_dir="${prefix}/${project_container}"
if [ -n "$project_parent_name" ];then
    project_dir+="/${project_parent_name}"
else
    project_dir+="/${project_name}"
fi
code 'project_dir="'$project_dir'"'
____

INDENT+="    " \
rcm-composer-autoinstaller $isfast \
    && INDENT+="    " \
rcm-drupal-autoinstaller-nginx $isfast \
    $is_auto_add_group \
    $is_no_sites_default \
    $is_drush_install \
    $is_composer_offline \
    --drupal-version="$drupal_version" \
    --drupalcms-version="$drupalcms_version" \
    --php-version="$php_version" \
    --php-fpm-section="$php_fpm_section" \
    --php-fpm-user="$php_fpm_user" \
    --project-dir="$project_dir" \
    --project-name="$project_name" \
    --project-parent-name="$project_parent_name" \
    --url-scheme="$url_scheme" \
    --url-host="$url_host" \
    --url-port="$url_port" \
    --url-path="$url_path" \
    ; [ ! $? -eq 0 ] && x

if [[ -n "$url" && -n "$certbot_obtain" ]];then
    INDENT+="    " \
    rcm-certbot-apt $isfast \
        ; [ ! $? -eq 0 ] && x
fi

if [ -n "$url" ];then
    INDENT+="    " \
    rcm-drupal-setup-wrapper-nginx-virtual-host-autocreate-php-multiple-root $isfast \
        $is_certbot_obtain \
        --php-version="$php_version" \
        --php-fpm-section="$php_fpm_section" \
        --project-dir="$project_dir" \
        --url-scheme="$url_scheme" \
        --url-host="$url_host" \
        --url-port="$url_port" \
        --url-path="$url_path" \
        --certificate-name="$certificate_name" \
        ; [ ! $? -eq 0 ] && x

    chapter Flush cache.
    code drush cache:rebuild --uri="$url"
    sudo -u "$php_fpm_user" PATH="${project_dir}/drupal/vendor/bin":$PATH $env -s \
        drush cache:rebuild --uri="$url"
    ____
fi

if [ -n "$url" ];then
    chapter Saving URL information.
    code mkdir -p '"'$url_dirname_website_info'"'
    mkdir -p "$url_dirname_website_info"
    cat << EOF >> "${url_dirname_website_info}/website"
URL_DRUPAL=$url
EOF
    fileMustExists "${url_dirname_website_info}/website"
    ____
fi

INDENT+="    " \
rcm-drupal-setup-drush-alias $isfast \
    --project-name="$project_name" \
    --project-parent-name="$project_parent_name" \
    --url-scheme="$url_scheme" \
    --url-host="$url_host" \
    --url-port="$url_port" \
    --url-path="$url_path" \
    && INDENT+="    " \
rcm-drupal-setup-internal-command-cd-drupal $isfast \
    && INDENT+="    " \
rcm-drupal-setup-internal-command-ls-drupal $isfast \
    && INDENT+="    " \
rcm-drupal-setup-internal-command-find-drupal $isfast \
    && INDENT+="    " \
rcm-drupal-setup-dump-variables $isfast \
    --project-name="$project_name" \
    --project-parent-name="$project_parent_name" \
    --domain="$domain" \
    ; [ ! $? -eq 0 ] && x

chapter Finish
____

exit 0

# parse-options.sh \
# --without-end-options-double-dash \
# --compact \
# --clean \
# --no-hash-bang \
# --no-original-arguments \
# --no-error-invalid-options \
# --no-error-require-arguments << EOF | clip
# FLAG=(
# --fast
# --version
# --help
# --no-sites-default
# --no-auto-add-group
# --no-drush-install
# --offline
# )
# VALUE=(
# --project-name
# --project-parent-name
# --url
# --php-fpm-user
# --prefix
# --project-container
# --variation
# --certificate-name
# )
# MULTIVALUE=(
# --php-fpm-config
# )
# FLAG_VALUE=(
# )
# CSV=(
    # 'long:--with-certbot-obtain,parameter:certbot_obtain'
    # 'long:--without-certbot-obtain,parameter:certbot_obtain,flag_option:reverse'
# )
# EOF
# clear
